package com.storage.service.impl;

import org.springframework.transaction.annotation.Transactional;
import org.slf4j.Logger;

import com.storage.model.EasyException;
import java.util.List;
import com.storage.service.StorageOrderService;
import net.sf.json.JSONObject;
import javax.annotation.Resource;
import com.storage.utils.MyUtils;

import com.storage.model.RequestParams;
import java.util.Map;
import org.springframework.data.redis.core.StringRedisTemplate;

import org.springframework.stereotype.Service;
import org.slf4j.LoggerFactory;

import com.storage.model.ResponseParams ;
import com.storage.config.dbConfig.JdbcTemplateDao;

/**
*@author jiushiwon
*
*/
@Service("storageOrderService")
public class StorageOrderServiceImpl implements StorageOrderService{

	@Resource
	private JdbcTemplateDao	jdbcTemplateDao;

	@Resource
	private StringRedisTemplate stringRedisTemplate;

	protected static Logger logger = LoggerFactory.getLogger(StorageOrderService.class;

	/**
	* 保存一条入库单
	*/
	@Override
	@Transactional
	public ResponseParams saveInOrder(RequestParams req) {
		if(MyUtils.isNull(req.getOrderInfo){
 			throw new EasyException(-1,"参数不能为空！");
		}
		JSONObject paramJson = JSONObject.fromObject(req.getOrderInfo());
		String in_type = paramJson.optString("in_type");
		Integer storage_id = paramJson.optInt("storage_id");
		if(MyUtils.isNull(storage_id)) {
			throw new EasyException(-1,"参数storage_id非空！");
		}
		Integer user_id = paramJson.optInt("user_id");
		if(MyUtils.isNull(user_id)) {
			throw new EasyException(-1,"参数user_id非空！");
		}
		Integer total_amount = paramJson.optInt("total_amount");
		if(MyUtils.isNull(total_amount)) {
			throw new EasyException(-1,"总金额不能为空！");
		}
		JSONArray order_details = paramJson.optJSONArray("order_details");
		ResponseParams  resp =new ResponseParams (); 
	 	try { 
			String sql0 = " INSERT INTO yun_storage_in_order(order_status,order_dtme,handel_user_id,storage_id,product_total_price,in_type,order_no) VALUES (0,now(),user_id,storage_id,total_amount,in_type,0)";
			Integer id0 =  jdbcTemplateDao.singleInsert(sql0);
			for(Object o : order_details){
				JSONObject js = JSONObject.fromObject(o);
				String sql1 = " INSERT INTO yun_storage_in_order_details(order_no,total_amount,product_id,in_num) VALUES (0,in_type,"+ js.opt("product_id")+","+ js.opt("product_num")+")";
				Integer id1 =  jdbcTemplateDao.singleInsert(sql1);
			}
			for(Object o : order_details){
				JSONObject js = JSONObject.fromObject(o);
				String sql2 = " INSERT INTO yun_storage_flow(add_num,product_id,order_type) VALUES ("+ js.opt("product_num")+","+ js.opt("product_id")+",入库单)";
				Integer id2 =  jdbcTemplateDao.singleInsert(sql2);
			}
	 	}catch(Exception e){
	 		throw new EasyException(-1,"操作失败！");
	 	}
		return resp; 
	}

	/**
	* 保存一条出库单，库存扣减，库存不能为负
	*/
	@Override
	@Transactional
	public ResponseParams saveOutOrder(RequestParams req) {
		if(MyUtils.isNull(req){
 			throw new EasyException(-1,"参数不能为空！");
		}
		JSONObject paramJson = JSONObject.fromObject(req);
		Integer user_id = paramJson.optInt("user_id");
		if(MyUtils.isNull(user_id)) {
			throw new EasyException(-1,"参数user_id非空！");
		}
		Integer product_id = paramJson.optInt("product_id");
		if(MyUtils.isNull(product_id)) {
			throw new EasyException(-1,"参数product_id非空！");
		}
		Integer product_num = paramJson.optInt("product_num");
		if(MyUtils.isNull(product_num)) {
			throw new EasyException(-1,"参数product_num非空！");
		}
		ResponseParams  resp =new ResponseParams (); 
	 	try { 
			String sql0 = " SELECT storage_num AS `storage_num` FROM yun_product_storage wrZ_Nrl WHERE  1=1  AND (product_id  =  "+productId+")";
			List<Map<String, Object>> conditionList0=jdbcTemplateDao.queryForList(sql0);
			Object storageNum=conditionList0.get(0).get("storage_num");
			if(storageNum<=product_num){
				throw new EasyException(-1,"库存不足");
			}
			String sql1 = " INSERT INTO yun_storage_out_order(order_status,order_dtme,handle_user_id) VALUES (1,now(),user_id)";
			Integer id1 =  jdbcTemplateDao.singleInsert(sql1);
			String sql2 = " INSERT INTO yun_storage_out_order_details(product_id,out_num) VALUES (product_id,product_num)";
			Integer id2 =  jdbcTemplateDao.singleInsert(sql2);
			String sql3 = " INSERT INTO yun_storage_flow(product_id,sub_num,order_type) VALUES (product_id,product_num,1)";
			Integer id3 =  jdbcTemplateDao.singleInsert(sql3);
	 	}catch(Exception e){
	 		throw new EasyException(-1,"操作失败！");
	 	}
		return resp; 
	}

	@Override
	@Transactional
	public ResponseParams getInOrder(RequestParams req) {
		if(MyUtils.isNull(req){
 			throw new EasyException(-1,"参数不能为空！");
		}
		JSONObject paramJson = JSONObject.fromObject(req);
		String orderNo = paramJson.optString("orderNo");
		if(orderNo==null||orderNo.length()==0) {
			throw new EasyException(-1,"参数orderNo非空！");
		}
		ResponseParams  resp =new ResponseParams (); 
	 	try { 
			if(orderNo==null){
				throw new EasyException(-1,"订单号不能为空！");
			}
			String sql0 = "null";
			List<Map<String, Object>> mapList0=jdbcTemplateDao.queryForList(sql0);
			resp.setExtra(mapList0); 
	 	}catch(Exception e){
	 		throw new EasyException(-1,"操作失败！");
	 	}
		return resp; 
	}

	@Override
	@Transactional
	public ResponseParams getOutOrder(RequestParams req) {
		if(MyUtils.isNull(req){
 			throw new EasyException(-1,"参数不能为空！");
		}
		JSONObject paramJson = JSONObject.fromObject(req);
		ResponseParams  resp =new ResponseParams (); 
	 	try { 
			String sql0 = " SELECT order_no AS `order_no`,product_total_price AS `product_total_price`,order_dtme AS `order_dtme`,handle_user_id AS `handle_user_id` FROM yun_storage_out_order Gjs_xrF WHERE  1=1 ";
			List<Map<String, Object>> mapList0=jdbcTemplateDao.queryForList(sql0);
			resp.setExtra(mapList0); 
	 	}catch(Exception e){
	 		throw new EasyException(-1,"操作失败！");
	 	}
		return resp; 
	}

	/**
	* 删除一条订单
	*/
	@Override
	@Transactional
	public ResponseParams deleteOrder(RequestParams req) {
		if(MyUtils.isNull(req){
 			throw new EasyException(-1,"参数不能为空！");
		}
		JSONObject paramJson = JSONObject.fromObject(req);
		String orderNo = paramJson.optString("orderNo");
		if(orderNo==null||orderNo.length()==0) {
			throw new EasyException(-1,"参数orderNo非空！");
		}
		ResponseParams  resp =new ResponseParams (); 
	 	try { 
			String sql0 = " UPDATE yun_storage_out_order Gjs_xrF SET order_status=0,last_updtme= now()  WHERE  1=1  AND (storage_id  =  "+orderNo+")";
			jdbcTemplateDao.executeSql(sql0);
	 	}catch(Exception e){
	 		throw new EasyException(-1,"操作失败！");
	 	}
		return resp; 
	}


}
